@@ -1,20 +1,11 @@
 import optparse
 import os
-import signal
 import subprocess
 import sys
 
 from .env import Env
 from .version import __version__
 
-# must have shell = True on Windows
-is_windows = sys.platform == 'win32'
-
-if is_windows:
-    params = {'creationflags': subprocess.CREATE_NEW_PROCESS_GROUP}
-else:
-    params = {'preexec_fn': os.setsid}
-
 
 class Response(Exception):
     def __init__(self, message='', status=0):
@@ -30,7 +21,6 @@ class Runner(object):
         self.parser = optparse.OptionParser(version=__version__)
         self.parser.disable_interspersed_args()
         self.parser.prog = 'envdir'
-        signal.signal(signal.SIGTERM, self.terminate)
 
     def path(self, path):
         real_path = os.path.realpath(os.path.expanduser(path))
@@ -45,7 +35,10 @@ class Runner(object):
     def open(self, path=None, stacklevel=1):
         if path is None:
             frame = sys._getframe()
-            get_parent = lambda frame: frame.f_back
+
+            def get_parent(frame):
+                return frame.f_back
+
             for _ in range(stacklevel):
                 frame = get_parent(frame)
             if frame is not None:
@@ -62,7 +55,7 @@ class Runner(object):
         options, args = self.parser.parse_args(list(args))
 
         if len(args) == 0:
-            raise Response("%s\nError: incorrect number of arguments" %
+            raise Response("%s\nError: incorrect number of arguments\n" %
                            (self.parser.get_usage()), 2)
 
         sys.stdout.write("Launching envshell for %s. "
@@ -74,14 +67,11 @@ class Runner(object):
         shell = os.environ['SHELL']
 
         try:
-            subprocess.check_call([shell],
-                                  universal_newlines=True,
-                                  bufsize=0,
-                                  close_fds=not is_windows,
-                                  **params)
+            subprocess.call([shell])
         except OSError as err:
             if err.errno == 2:
-                raise Response("Unable to find shell %s" % shell, err.errno)
+                raise Response("Unable to find shell %s" % shell,
+                               status=err.errno)
             else:
                 raise Response("An error occurred: %s" % err,
                                status=err.errno)
@@ -107,35 +97,9 @@ class Runner(object):
             args = args[1:]
 
         try:
-            self.process = subprocess.Popen(args,
-                                            universal_newlines=True,
-                                            bufsize=0,
-                                            close_fds=not is_windows,
-                                            **params)
-            self.process.wait()
+            os.execvpe(args[0], args, os.environ)
         except OSError as err:
-            if err.errno == 2:
-                raise Response("Unable to find command %s" %
-                               args[0], err.errno)
-            else:
-                raise Response(status=err.errno)
-        except KeyboardInterrupt:
-            self.terminate()
-        raise Response(status=self.process.returncode)
+            raise Response("Unable to run command %s: %s" %
+                           (args[0], err), status=err.errno)
 
-    def terminate(self, *args, **kwargs):
-        # first send mellow signal
-        self.quit(signal.SIGTERM)
-        if self.process.poll() is None:
-            # still running, kill it
-            self.quit(signal.SIGKILL)
-
-    def quit(self, signal):
-        if self.process.poll() is None:
-            proc_pgid = os.getpgid(self.process.pid)
-            if os.getpgrp() == proc_pgid:
-                # Just kill the proc, don't kill ourselves too
-                os.kill(self.process.pid, signal)
-            else:
-                # Kill the whole process group
-                os.killpg(proc_pgid, signal)
+        raise Response()
